﻿using UnityEngine;
using System.Collections.Generic;

/// <summary>
/// The cut manager is assigned to a Geometry as a script.
/// the user can choose if the geometry is cutted using a Window Cut or a Box Cut.
/// </summary>
public class CutManager : MonoBehaviour {

	public List<Cut> cutPerViewpoint = new List<Cut>();
	private string cutName = "";
	private string[] cutChoices = new[] {"Window Cut", "Box Cut"};
	void Start(){
		transform.tag = "CutObject";
		cutName = getCutName ();
		setCut ();
	}
	
	/// <summary>
	/// Creates Window cut or a box cut and adds the cut to each viewpoint in the scene. 
	/// </summary>
	public void setCut(){
		
		cutPerViewpoint = new List<Cut> ();
		for (int i = 0; i < GameObject.Find("Main").GetComponent<Main>().getCameraCount(); i++) {	
			if (cutName.Equals ("Window Cut")) {
				cutPerViewpoint.Add(new FreeformWindowCut (transform));
			} else if (cutName.Equals ("Box Cut")) {
				cutPerViewpoint.Add(new BoxCut(transform));
			}
		}
	}

	/// <summary>
	/// Retunrs the i-th cut of the <c>ViewPoint</c>
	/// </summary>
	/// <param name="i">The index of the cut in the <c>ViewPoint</c> List </param>
	/// <returns>Eigher a <c>BoxCut</c> or a <c>FreeformWindowCut</c></returns>
	public Cut getCut(int i){
		return cutPerViewpoint[i];
	}

	/// <summary>
	/// Returns if the <c>ViewPoint</c> has any cuts
	/// </summary>
	/// <returns>true if the <c>ViewPoint</c> contains cuts, false if it don't</returns>
	public bool hasCuts(){
		return cutPerViewpoint.Count > 0;
	}

	/// <summary>
	/// Sets the name of the cut, to recall it later
	/// </summary>
	/// <param name="name">The name of the cut</param>
	public void setCutName(string name){
		this.cutName = name;
	}

	/// <summary>
	/// Returns the name of the geometry object
	/// </summary>
	/// <returns>Name of geometry object</returns>
	public string getNameOfGeo(){
		if(transform != null){
			return transform.name;
		}
		return "";
	}

	/// <summary>
	/// Returns the cut name
	/// </summary>
	/// <returns>Cut name</returns>
	private string getCutName(){
		if (cutName.Equals ("")) {
			string keyName = transform.name + "_CM_choincesIndex";
			if (PlayerPrefs.HasKey (keyName)) {
				this.cutName = cutChoices [PlayerPrefs.GetInt (keyName)];
			}
		}
		return cutName;
	}
}
