﻿using UnityEngine;
using System.Collections.Generic;

/// <summary>
/// This is the main script of the program. It contains the name of the current target Object and a list of the cameras in the scene.
/// </summary>
public class Main : MonoBehaviour {

	public static string tagName = "CutObject";
    //public static List<GameObject> currentTargets = new List<GameObject>();
    public static GameObject[] currentTarget;
    public GameObject[] cameras = new GameObject[3];
	private List<ViewPoint> viewPointList = new List<ViewPoint>();
	private int activeVPIndex = 0;

	/// <summary>
	/// This function is executed at the beginning of the program.
	/// </summary>
	void Start(){
	}

	/// <summary>
	/// The function creates a viewpoint for each camera in the scene and adds cuts to the viewpoints.
	/// </summary>
	public void computeViewPoints(){
		Debug.Log ("Compute Viewpoints. Count: " + cameras.Length);
		viewPointList = new List<ViewPoint> ();
		for (int i = 0; i < cameras.Length; i++) {
			viewPointList.Add (new ViewPoint(cameras[i]));
		}
		addCutsToViewpoints ();
	}

	/// <summary>
	/// For each viewpoint an occlusion graph is computed
	/// </summary>
	public void computeOcclusionGraph(){
		for (int i = 0; i < viewPointList.Count; i++) {
			viewPointList [i].computeOcclusionGraph ();
		}
	}

	/// <summary>
	/// Sets the name of the target element. The function looks for the corresponding target and saves the geometry of it.
	/// </summary>
	/// <param name="targetName">Name of the target</param>
	public void setCurrentTarget(string targetName){
		Target[] targets = GetComponents<Target> ();
		bool targetFound = false;
		for (int i = 0; i < targets.Length; i++) {
			if (targetName.Equals (targets[i].TargetName)) {
				targetFound = true;
                currentTarget = targets[i].targetObject;
                /*for (int j = 0; j < targets [i].TargetObjects.Length; j++) {
					currentTargets.Add (targets [i].TargetObjects[j]);
				}*/
			}
		}
		if (!targetFound) {
			Debug.LogError ("Kein Target mit Name " + targetName + " definiert.");
		}
	}
	
	/// <summary>
	/// Precomputes cuts for each viewpoint for each geometry that contains a cutManager script.
	/// </summary>
	public void precomputeCuts(){
        List<Transform> targets = new List<Transform>();
        for (int i = 0; i < currentTarget.Length; i++)
        {
            targets.Add(currentTarget[i].transform);
        }
		for (int i = 0; i < viewPointList.Count; i++) {
			viewPointList [i].precomputeCuts (targets);
		}
	}

	/// <summary>
	/// Performs the final cut on each geometry with a cutManager script for the current viewpoint.
	/// </summary>
	public void performFinalCut(){
		Debug.Log(activeVPIndex);
		//Debug.Log(viewPointList [activeVPIndex]);
		//Debug.Log(currentTarget);
        List<Transform> targets = new List<Transform>();
        for (int i = 0; i < currentTarget.Length; i++)
        {
            targets.Add(currentTarget[i].transform);
        }
        viewPointList[activeVPIndex].performFinalCut(targets);
    }


	/// <summary>
	/// Sets the active viewpoint to the index.
	/// </summary>
	/// <param name="i">Index of current viewpoint</param>
    public void setActiveViewpoint(int i){
        if(i < 0 || viewPointList[i] == null)
        {
            Debug.LogError("Es gibt zu keine Kamera an Position " + i);
        }
		Debug.Log ("Set active Viewpoint nr. " + i);
		viewPointList [activeVPIndex].activateCamera (false);
		activeVPIndex = i;
		viewPointList [activeVPIndex].activateCamera (true);
	}

	/// <summary>
	/// Saves a screenshot in the unity folder with a specific name.
	/// </summary>
	/// <param name="name">The name of the screenshot</param>
    public void makeScreenshot(string name)
    {
        if (name.Equals("")) { 
            Application.CaptureScreenshot("Screenshot.png");
        }
        else
        {
            if (!name.EndsWith(".png")) name += ".png";
            Application.CaptureScreenshot(name);
        }
    }

	/// <summary>
	/// Adds an instance of a cut to each viewpoint. Each viewpoint has its own cuts.
	/// </summary>
	private void addCutsToViewpoints(){
	//	Debug.Log ("Add an instance of a cut To a viewPoint");
		GameObject[] allCutManagers = GameObject.FindGameObjectsWithTag (tagName); 

		for (int i = 0; i < allCutManagers.Length; i++) {
			for (int j = 0; j < viewPointList.Count; j++) {
				viewPointList [j].addCut (allCutManagers[i].GetComponent<CutManager>().getCut(j));	
			}
		}
	}

	/// <summary>
	/// Returns the ammount of viewpoints
	/// </summary>
	/// <returns>Count of the camera List</returns>
	public int getCameraCount(){
		return cameras.Length;
	}

	/// <summary>
	/// Resets the geometry of all objects
	/// </summary>
	public void resetGeometry()
	{
		foreach (var vp in viewPointList)
		{
			vp.resetGeometries();
		}
	}
}
